#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdio.h>
/* the bitmap file "theIcon" is a file made from the X11
** program bitmap, which is a general-purpose bitmap editor.
** You can use the bitmap file as presented, or create your
** own. */
#include "theIcon"
#define      BORDER_WIDTH     10
/*           Programwide Globals                          */

Display      *theDisplay;  /* Which Display               */
int          theScreen;    /* Which screen on the display */
int          theDepth;     /* Number of color planes      */
int          xl = 0;    /* remember last position */
int          yl = 0;    /* remember last position */
int          chtl;    /* remember last fontheight*/
unsigned long   theBlackPixel; /*System black color*/
unsigned long   theWhitePixel; /*System white color*/
Window   theWindow;
GC       theGC;
XFontStruct *fontStruct;
char        *fontname;
/* gistart_() - Initalization of X window*/

void gistart_(x,y,width,height)

int          *x,*y;          /* Where the window should go  */
int          *width,*height; /* Size of the window          */

{
/*      printf("gistart");*/
/* 1) Establish a connection to the X server. The connection
**    is asked for on the local server for local display. */

   theDisplay = XOpenDisplay(NULL);

/*  Check if the connection was made   */

   if (theDisplay == NULL)
      {
      printf(stderr,
"ERROR: Cannot establish a connection to the X server%s\n",
             XDisplayName(NULL));
      exit(1);
      }

/* 2) Check for the default screen and color plane depth.
**    If theDepth == 1, then we have a monochrome system  */

   theScreen = DefaultScreen(theDisplay);
   theDepth  = DefaultDepth(theDisplay,theScreen);
   theBlackPixel = BlackPixel(theDisplay,theScreen);
   theWhitePixel = WhitePixel(theDisplay,theScreen);
/* 3) Get some info on the display and print it.          */
/*   getXInfo();*/
/* 4) Open a window on the display.                       */
   theWindow = openWindow(*x,*y,         /* x,y           */
                          *width,*height,/* width,height  */
                          0,           /* not pop-up      */
                          &theGC);     /* Graphics Context*/
/* 5) Select font for text.                 */
   chtl = 0;
   fontname = "*-*-medium-r-normal--8-*-*-*-c-*";
   initFont();
} /* function gistart_() */

/* getXInfo prints information to the starting terminal
** (usually an xterm) about the current X Window display
** and screen. */

getXInfo()
{
/* You will find many funny values when you check the X
** information below. It should be something to the tune of:
** MIT X Consortium (vendor)
** 3 (vendor release)
** X11 R3 (or R2, the Version and Revision numbers)
** But, on most implementations, this information does not
** seem to jibe.
** On the generic MIT X Consortium release for X11R3, you
** might see X11R0, with a vendor release # of 3.  */

   printf("%s version %d of the X Window System, X%d R%d\n",
          ServerVendor(theDisplay),VendorRelease(theDisplay),
          ProtocolVersion(theDisplay),
          ProtocolRevision(theDisplay));
/* Now, print some more detailed information
** about the display. */

   if (theDepth == 1)
      {
      printf("Color plane depth...........%d (Monochrome)\n",
             theDepth);
      }
   else
      {
      printf("Color plane depth...........%d\n",theDepth);
      }
   printf("Display Width...............%d\n",
          DisplayWidth(theDisplay,theScreen));
   printf("Display Height..............%d\n",
          DisplayHeight(theDisplay,theScreen));
   printf("The display %s\n",XDisplayName(theDisplay));
}    /* function getXInfo    */
/* openWindow
**
** This function takes a x, y pixel location for the upper
** left corner of the window, as well as a width and height
** for the for the size of the window. It opens a window on
** the X display, and returns the window id. It is also pas-
** sed a flag that specifies whether the window is to be a
** pop-up window (such as a menu) or not. This initial ver-
** sion does not implement that feature.
** The GC theNewGC is a graphics context that is created for
** the window, and returned to the caller. The GC is necessary
** for drawing into the window.
**
** 1) Set up the attributes desired for the window.
** 2) Open a window on the display.
** 3) Set up an Icon for the window.
** 4) Send "Hints" to the Window Manager.
** 5) Now tell the window manager about the size and loca-
**    tion of the window.
** 6) Create a graphics context for the window.
** 7) Ask X to place the window visibly on the screen.
** 8) Flush out all the queued up X requests to the X
**    server.
**
** Note: With this program, you can either be friendly or
** unfriendly to the window manager, if one is running.
** See the section on override_redirect below. If you set
** override_redirect to True, you are being unfriendly to
** the window manager.
*/

openWindow(x,y,width,height,flag,theNewGC)

int          x,y;          /* Where the window should go  */
int          width,height; /* Size of the window          */
int          flag;         /* if > 0 then the window is
                              a pop up                    */
GC           *theNewGC;    /* Returned Graphics Context   */

{                          /* Function openWindow         */
   XSetWindowAttributes  theWindowAttributes;
   XSizeHints            theSizeHints;
   unsigned  long        theWindowMask;
   Window                theNewWindow;
   Pixmap                theIconPixmap;
   XWMHints              theWMHints;

/* 1) Set up the attributes desired for the window.
** Note that window managers may deny us some of these
** resources. Note also that setting override_redirect
** to True will tell the window manager to leave our
** window alone. Try this program by setting
** override_redirect to False.
*/
   theWindowAttributes.border_pixel = theBlackPixel;
   theWindowAttributes.background_pixel = theWhitePixel;
/*   theWindowAttributes.override_redirect = True; */
   theWindowAttributes.override_redirect = False;
   theWindowMask =
       CWBackPixel|CWBorderPixel|CWOverrideRedirect;

/*
** 2) Open a window on the display.
*/

   theNewWindow =XCreateWindow(theDisplay,
        RootWindow(theDisplay,theScreen),
        x,y,width,height,BORDER_WIDTH,theDepth,
        InputOutput,CopyFromParent,theWindowMask,
        &theWindowAttributes);

/* 3) Set up an icon for the window. Each window should
**    also register an icon with the window manager.
**    This icon is used if the window is shrunk down to
**    an iconic form by the user (through interaction with
**    the window manager). */

   theIconPixmap = XCreateBitmapFromData(theDisplay,
                   theNewWindow,theIcon_bits,
                   theIcon_width,theIcon_height);

/* 4) Send "Hints" to the Window Manager. Before this
**    window will appear on the display, an X window
**    manager may intercept the call and place the window
**    where it wants to. This next section tells the win-
**    dow manager "hints" as to where the window should go.
*/
   theWMHints.icon_pixmap   = theIconPixmap;
   theWMHints.initial_state = NormalState;
   theWMHints.flags         = IconPixmapHint|StateHint;

   XSetWMHints(theDisplay,theNewWindow,&theWMHints);
/*
** 5) Now tell the window manager about the size and loca-
**    tion of the window.
*/

   theSizeHints.flags   = PPosition|PSize;
   theSizeHints.x       = x;
   theSizeHints.y       = y;
   theSizeHints.width   = width;
   theSizeHints.height  = height;
   XSetNormalHints(theDisplay,theNewWindow,&theSizeHints);

/* 6) Create a graphics context for the window.   */

   if (createGC(theNewWindow,theNewGC) == 0)
      {
      XDestroyWindow(theDisplay,theNewWindow);
      return((Window) 0);
      }
 
/*
** 7) Ask X to place the window visibly on the screen.
**    Up to now, the window has been created but has not
**    appeared on the screen. Mapping the window places
**    it visibly on the screen.
*/
   XMapWindow(theDisplay,theNewWindow);
/*
** 8) Flush out all the queued up X requests to the X
**    server.
*/
   XFlush(theDisplay);
/*
** 9) Return the window ID, which is needed to specify
**    which window to draw to.
*/

   return(theNewWindow);

}                          /* End of function openWindow  */

/* createGC creates a graphics context for the given window.
** A graphics context is necessary to draw into the window.
**
** Returns 0 if there was an error, 1 if all is OK.       */

createGC(theNewWindow,theNewGC)

Window     theNewWindow;
GC         *theNewGC;

{                          /* function createGC           */
   XGCValues      theGCValues;
   *theNewGC = XCreateGC(theDisplay,theNewWindow,
                         (unsigned long) 0,&theGCValues);
   if (*theNewGC == 0)     /* unable to create a GC       */
      {
      return(0);            /* Error  */
      }
   else
      {
   /* Set foregound and background defaults for the new GC*/
      XSetForeground(theDisplay,*theNewGC,theBlackPixel);
      XSetBackground(theDisplay,*theNewGC,theWhitePixel);
      return(1);            /* OK  */
      }
} /* function createGC           */

/* initFont()
** Initializes a font for drawing text in for a given gra-
** phics context. If you just use this font for a given GC,
** then you no longer have to worry about the font.       */

initFont()

{        /* function initFont   */

/*   XFontStruct *fntStrct; */
   fontStruct = XLoadQueryFont(theDisplay,fontname);
   if (fontStruct != 0)
      {
      XSetFont(theDisplay,theGC,fontStruct->fid);
      }
   else
      {
      printf("Could not load Font\n");
      }

/*   fntStrct = fontStruct; */
/*    return(fntStrct); */
}        /* function initFont   */

/* quitx_() - closes the connection to the X server        */

void quitx_()

{     /* Function quitx              */
/*   sleep(3);*/
   XDestroyWindow(theDisplay,theWindow);
   XFreeFont(theDisplay,fontStruct);
   XCloseDisplay(theDisplay);
}     /* End of function quitx       */
/* drawLine
** Draws a line from (x1,y1) to (x2,y2) in the window
** theWindow, using the graphics context theGC.           */

void drawline_(xt,yt,act)
   int        *xt,*yt;        /* Ending location          */
   int        *act;   /* 2 => draw, 3 => move */
{  /* function drawLine   */
   if (*act == 2)
     {
     XDrawLine(theDisplay,theWindow,theGC,xl,yl,*xt,*yt);
     XFlush(theDisplay);
      }
   xl = *xt;    /* Remeber last location        */
   yl = *yt;

}  /* function drawLine   */

/* write one character on screen */
void wrchar_(x,y,a,cheight)
   int        *x,*y;  /*lower left location of characterbox*/
   char       *a;   /*character to be written on the screen*/
   int        *cheight; /*characterbox height*/
{  /* function wrchar_ */
/*   int i;*/
/*      int        h;*/
/*       h = fontStruct->ascent;*//*+ fontStruct->descent;*/
/*       printf("characterbox height as realized = %d\n",h);*/
/*  Select font for text.                 */
   if (*cheight != chtl)
    {
/*    printf("chtl = %d\n",chtl);
    printf("cheight = %d\n",*cheight);*/
    chtl = *cheight;
    if (chtl<=8)
     fontname = "*-fixed-medium-r-normal--8-*-*-*-c-*";
    else {
     if (chtl==9)
      fontname = "*-fixed-medium-r-normal--9-*-*-*-c-*";
     else {
      if ((chtl>=10)&&(chtl<=12))
       fontname = "*-fixed-medium-r-normal--10-*-*-*-c-*";
      else {
       if ((chtl==13)||(chtl==14))
        fontname = "*-fixed-medium-r-normal--13-*-*-*-c-*";
       else {
        if ((chtl>=15)&&(chtl<=19))
         fontname = "*-fixed-medium-r-normal--15-*-*-*-c-*";
        else {
         if ((chtl>=20)&&(chtl<=23))
          fontname = "*-fixed-medium-r-normal--20-*-*-*-c-*";
         else
          fontname = "*-fixed-medium-r-normal--24-*-*-*-c-*";
             }
            }
           }
          }
         }
/*    printf("fontname = %s\n",fontname);*/
    initFont();
    }

/*"variable";*/
   XDrawImageString(theDisplay,theWindow,theGC,*x,*y,a,1);
/* Send output tothe screen */
   XFlush(theDisplay);
}  /* function wrchar_ */


